/******************************************************************************
 * Copyright (C) 2018  Tadeus Prastowo <eus@member.fsf.org>                   *
 *                                            (GPG public key ID: 0x277B48A6) *
 *                                                                            *
 * This program is free software: you can redistribute it and/or modify       *
 * it under the terms of the GNU General Public License as published by       *
 * the Free Software Foundation, either version 3 of the License, or          *
 * (at your option) any later version.                                        *
 *                                                                            *
 * This program is distributed in the hope that it will be useful,            *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of             *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the              *
 * GNU General Public License for more details.                               *
 *                                                                            *
 * You should have received a copy of the GNU General Public License          *
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.      *
 ******************************************************************************/

namespace tice {
  namespace v1 {
    namespace internals {
      namespace node {
        
        template<typename... args>
        struct check_arg_1 : std::false_type {
        };

        template<std::intmax_t... ratio_args, typename... args>
        struct check_arg_1<std::ratio<ratio_args...>, args...> : std::true_type {
        };

      };
      namespace node {
        
        template<typename... args>
        struct check_arg_2 : std::false_type {
        };

        template<typename arg__1, typename... comp_args, typename... args>
        struct check_arg_2<arg__1, comp::Comp<comp_args...>, args...> : std::true_type {
        };

      };
      namespace node {

        template<typename... args>
        struct check_wcet_and_period : std::false_type {
        };

        template<typename period, typename wcet, typename... comp_args, typename... args>
        struct check_wcet_and_period<period, comp::Comp<wcet, comp_args...>, args...>
          : std::integral_constant<bool, std::ratio_greater_equal<period, wcet>::value> {
        };
        
      };
    };
  };
};

namespace tice {
  namespace v1 {

    template<typename arg__Comp__computation, typename arg__ratio__period>
    class Node {
      static_assert(internals::node::check_arg_1<arg__ratio__period, arg__Comp__computation>::value,
                    "Period is not specified using class template tice::v1::Ratio");
      static_assert(internals::node::check_arg_2<arg__ratio__period, arg__Comp__computation>::value,
                    "Computation is not specified using class template tice::v1::internals::comp::Comp");
      static_assert(internals::node::check_wcet_and_period<arg__ratio__period, arg__Comp__computation>::value,
                    "Period is less than computation WCET");
    public:
      typedef arg__ratio__period period;
      typedef typename arg__Comp__computation::wcet wcet;
      typedef typename arg__Comp__computation::fn_info fn_info;
    };
    
  };
};
