/******************************************************************************
 * Copyright (C) 2018  Tadeus Prastowo <eus@member.fsf.org>                   *
 *                                            (GPG public key ID: 0x277B48A6) *
 *                                                                            *
 * This program is free software: you can redistribute it and/or modify       *
 * it under the terms of the GNU General Public License as published by       *
 * the Free Software Foundation, either version 3 of the License, or          *
 * (at your option) any later version.                                        *
 *                                                                            *
 * This program is distributed in the hope that it will be useful,            *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of             *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the              *
 * GNU General Public License for more details.                               *
 *                                                                            *
 * You should have received a copy of the GNU General Public License          *
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.      *
 ******************************************************************************/

#include <type_traits>
#include <ratio>
#include <tuple>
#include <utility>
#include <cstdint>

namespace tice {
  namespace v1 {

    //\begin{data structures}

    template<std::intmax_t numerator, std::intmax_t denominator = 1>
    using Ratio = std::ratio<numerator, denominator>;

    template<typename element_1, typename element_2>
    using Pair = std::pair<element_1, element_2>;

    template<int... args__core_id>
    using Core_ids = std::tuple<std::integral_constant<int, args__core_id>...>;

    //\end{data structures}



    //\begin{architecture-dependent components}

    // Hardware description.
    template<typename arg__core_ids>
    class HW;
    
    // Computation.  No computation may share an arg__function_ptr since
    // the pointer is used internally as a node ID to prevent duplicated nodes.
    #define Comp(arg__function_ptr, arg__ratio__wcet...) \
      tice::v1::internals::comp::Comp<arg__ratio__wcet, std::integral_constant<decltype(arg__function_ptr), arg__function_ptr>>

    //\end{architecture-dependent components}



    //\begin{architecture-independent components}

    // A node in a Tice graph.
    template<typename arg__Comp__computation, typename arg__ratio__period>
    class Node;

    // A channel whose initial value is specified inline.
    template<typename arg__channel_type, arg__channel_type arg__initial_value>
    class Chan_inline_init;

    // A channel whose initial value cannot be or is not specified inline.
    // E.g., type double cannot have its value specified as a nontype nontemplate argument.
    template<typename arg__channel_type, std::add_pointer_t<std::add_const_t<arg__channel_type>> arg__initial_value_ptr>
    class Chan;

    // A set of the incoming arcs of a node, which is called a consumer, in a Tice graph.
    template<typename arg__Node__consumer, typename arg__pair__of__Chan_inline_init_or_Chan__channel__and__Node__producer,
             typename... args__pair__of__Chan_inline_init_or_Chan__channel__and__Node__producer>
    class Feeder;

    // An end-to-end delay constraint decorating a Tice graph.
    template<typename arg__Node__sensor_node, typename arg__Node__actuator_node,
             typename arg__ratio__min_delay, typename arg__ratio__max_delay>
    class ETE_delay;

    // A correlation constraint decorating a Tice graph.
    template<typename arg__Node__actuator_node, typename arg__ratio__threshold, typename arg__Node__sensor_node,
             typename... args__Node__sensor_node>
    class Correlation;

    // A program implementing a Tice graph.
    template<typename... args__HW__then__Node__then__Feeder__then__ETE_delay__then__Correlation>
    class Program;

    //\end{architecture-independent components}

  };
};

#include "internals/v1/v1_internals.hpp"
